import pytest

from chalk.features import feature, features, unwrap_feature


@features
class CustomNameClass:
    id: str
    other: str = feature(name="other_name")


def test_custom_name():
    assert CustomNameClass(other="papaya").other == "papaya"
    assert CustomNameClass(id="4", other="papaya") == CustomNameClass(id="4", other="papaya")
    assert unwrap_feature(CustomNameClass.other).name == "other_name"
    assert str(CustomNameClass(other="papaya")) == "CustomNameClass(other=papaya)"
    assert str(CustomNameClass(id="4", other="papaya")) == "CustomNameClass(id=4, other=papaya)"
    assert dict(CustomNameClass(id="4", other="papaya")) == {
        "custom_name_class.id": "4",
        "custom_name_class.other_name": "papaya",
    }
    assert len(CustomNameClass(id="4", other="papaya")) == 2
    assert len(CustomNameClass(id="4")) == 1


@features(name="prince")
class TheArtistFormerlyKnownAsPrince:
    id: str
    favorite_color: str = feature(name="fav")


def test_custom_namespace():
    assert TheArtistFormerlyKnownAsPrince(id="a").id == "a"
    assert TheArtistFormerlyKnownAsPrince(id="4", favorite_color="purple") == TheArtistFormerlyKnownAsPrince(
        id="4", favorite_color="purple"
    )
    assert str(TheArtistFormerlyKnownAsPrince.id) == "prince.id"
    assert str(TheArtistFormerlyKnownAsPrince.favorite_color) == "prince.fav"
    with pytest.raises(ValueError):

        @features(name="with spaces")
        class Spaced:
            id: int

    with pytest.raises(ValueError):

        @features(name="WithCaps")
        class CapitalizedCls:
            id: int



from __future__ import annotations

# NOTE: The functions in this file are not tested in chalkpy
# The integration tests in engine/ validate the behavior of load_dataset
import base64
import json
from concurrent.futures import Future, ThreadPoolExecutor
from enum import IntEnum
from typing import TYPE_CHECKING, Any, Dict, List, Mapping, Optional, Sequence, Union

from chalk.utils.missing_dependency import missing_dependency_exception

if TYPE_CHECKING:
    import polars as pl

from chalk.features import Feature, FeatureWrapper, ensure_feature
from chalk.features.pseudofeatures import CHALK_TS_FEATURE
from chalk.serialization.codec import FEATURE_CODEC

_DEFAULT_EXECUTOR = ThreadPoolExecutor(16)


class DatasetVersion(IntEnum):
    """Format of the parquet file. Used when loading a dataset so we know what format it is in"""

    # This is the format that bigquery dumps to when specifying an output bucket and output format
    # as part of an (async) query job
    # The output contains extra columns, and all column names are b32 encoded, because
    # bigquery does not support '.' in column names.
    # The client will have to decode column names before loading this data
    # All data, except for feature times, are json encoded
    BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES = 1

    # This is the format returned by the dataset writer in engine/
    DATASET_WRITER = 2

    # This format uses separate columns for the observed at and timestamp columns
    # The observed at column is the actual timestamp from when the observation was observed,
    # whereas the timestamp column is the original timestamp that the user requested
    BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES_V2 = 3

    COMPUTE_RESOLVER_OUTPUT_V1 = 4


def _parallel_download(uris: List[str], executor: ThreadPoolExecutor) -> pl.DataFrame:
    try:
        import polars as pl
    except ImportError:
        raise missing_dependency_exception("chalkpy[polars]")

    df_futures: list[Future[pl.DataFrame]] = []
    for uri in uris:
        df_futures.append(executor.submit(pl.read_parquet, uri))

    dfs = [df.result() for df in df_futures]
    dfs = [x.select(sorted(x.columns)) for x in dfs]
    df = pl.concat(dfs)
    return df


def _load_dataset_from_chalk_writer(
    uris: List[str],
    features: Sequence[Feature],
) -> pl.DataFrame:
    try:
        import polars as pl
    except ImportError:
        raise missing_dependency_exception("chalkpy[polars]")
    # V1 datasets should contain just a single URI
    # This URI can be read directly
    # We need to filter the features to remove any pseudofeatures
    if len(uris) != 1:
        raise ValueError("v1 datasets should have just a single URI")
    df = pl.read_parquet(uris[0])
    df = _filter_features(df, features)
    return df


def _filter_features(df: pl.DataFrame, features: Sequence[Feature]):
    # Select only the requested columns
    for f in features:
        if str(f) not in df.columns:
            actual_columns = ", ".join(df.columns)
            raise ValueError(f"Column {str(f)} is not in the dataset. The dataset has columns {actual_columns}")
    df = df.select([str(f) for f in features])
    return df


def _decode_col_name(col_name: str) -> str:
    x_split = col_name.split("_")
    if x_split[0] == "ca":
        assert len(x_split) == 2
        return x_split[1]
    elif x_split[0] == "cb":
        root_fqn_b32 = x_split[1]
        return base64.b32decode(root_fqn_b32.replace("0", "=").upper()).decode("utf8") + "_".join(x_split[2:])
    else:
        raise ValueError(f"Unexpected identifier: {x_split[0]}")


def _decode_column_names(column_names: List[str], ts_fqn: Optional[str]) -> Mapping[str, str]:
    ans: Dict[str, str] = {"__id__": "__id__"}
    found_ts = False
    for x in column_names:
        if x.endswith("__"):
            if x in ("__observed_at__", "__ts__", CHALK_TS_FEATURE.fqn) and ts_fqn is not None and not found_ts:
                ans[x] = ts_fqn
                found_ts = True
            continue
        ans[x] = _decode_col_name("_".join(x.split("_")[:2]))
    return ans


def _decode_column_names_bigquery_v2(column_names: List[str], ts_fqn: Optional[str]) -> Mapping[str, str]:
    ans: Dict[str, str] = {"__id__": "__id__", "__ts__": "__ts__"}
    for x in column_names:
        if x.endswith("__"):
            if x == "__oat__" and ts_fqn is not None:
                ans[x] = ts_fqn
            continue
        ans[x] = _decode_col_name("_".join(x.split("_")[:2]))
    return ans


def _json_decode(x: Optional[str]):
    if x is None:
        return None
    return json.loads(x)


def _json_encode(x: Optional[Any]):
    if x is None:
        return None
    return json.dumps(x)


def _load_dataset_bigquery(
    uris: List[str],
    features: Sequence[Feature],
    executor: Optional[ThreadPoolExecutor],
    version: DatasetVersion,
):
    try:
        import polars as pl
    except ImportError:
        raise missing_dependency_exception("chalkpy[polars]")
    # V2 datasets are in multiple files, and have column names encoded
    # due to DB limitations (e.g. bigquery does not support '.' in column names)
    # In addition, the datasets may contain extra columns (e.g. replaced observed at)
    # All values are JSON encoded
    if executor is None:
        executor = _DEFAULT_EXECUTOR
    df = _parallel_download(uris, executor)

    ts_fqn = next((f.fqn for f in features if f.is_feature_time), None)
    if version == DatasetVersion.BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES:
        decoded_col_names = _decode_column_names(
            df.columns,
            ts_fqn=ts_fqn,
        )
        # Filter out the __observed_at__ and __replaced_observed_at__ columns
        df = df.select(list(decoded_col_names.keys()))

        # Base32-decode the column names
        df = df.rename(dict(decoded_col_names))
    elif version == DatasetVersion.BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES_V2:
        decoded_col_names = _decode_column_names_bigquery_v2(
            df.columns,
            ts_fqn=ts_fqn,
        )

        # Filter out the __observed_at__ and __replaced_observed_at__ columns
        df = df.select(list(decoded_col_names.keys()))

        # Base32-decode the column names
        df = df.rename(dict(decoded_col_names))

        # Use the __ts__ column if the __oat__ column is null
        if ts_fqn is not None:
            df = df.with_columns(
                pl.when(pl.col(ts_fqn).is_null()).then(pl.col("__ts__")).otherwise(pl.col(ts_fqn)).alias(ts_fqn)
            )
        # Drop the "__ts__ column"
        df = df.drop("__ts__")
    elif version == DatasetVersion.COMPUTE_RESOLVER_OUTPUT_V1:
        unique_features = set(df.select(pl.col("feature_name").unique())["feature_name"].to_list())
        cols = [pl.col("value").filter(pl.col("feature_name") == fqn).first().alias(fqn) for fqn in unique_features]

        df = df.groupby("pkey").agg(cols)
        decoded_stmts = []
        for col in df.columns:
            if col == "pkey":
                continue
            if col == ts_fqn:
                decoded_stmts.append(pl.col(col).dt.with_time_zone("UTC"))
            else:
                decoded_stmts.append(
                    pl.col(col).apply(_json_decode, return_dtype=FEATURE_CODEC.get_polars_dtype(fqn=col))
                )
        df = df.select(decoded_stmts)
        return df.select(sorted(df.columns))

    else:
        raise ValueError(f"Unsupported version: {version}")
    # Select only the requested columns
    for f in features:
        if str(f) not in df.columns:
            if f.primary:
                # Using the "__id__" column
                df = df.with_column(
                    pl.col("__id__")
                    .apply(_json_encode, return_dtype=FEATURE_CODEC.get_polars_dtype(f.fqn))
                    .alias(f.fqn)
                )
            else:
                actual_columns = ", ".join(df.columns)
                raise ValueError(f"Column {str(f)} is not in the dataset. The dataset has columns {actual_columns}")
    df = df.select([str(f) for f in features])

    # The parquet file is all JSON-encoded
    decoded_stmts = []
    for col in df.columns:
        if col == ts_fqn:
            decoded_stmts.append(pl.col(col).dt.with_time_zone("UTC"))
        else:
            decoded_stmts.append(pl.col(col).apply(_json_decode, return_dtype=FEATURE_CODEC.get_polars_dtype(fqn=col)))

    df = df.select(decoded_stmts)
    return df.select(sorted(df.columns))


def load_dataset(
    uris: List[str],
    version: DatasetVersion,
    features: Sequence[Union[str, Feature, FeatureWrapper, Any]],
    executor: Optional[ThreadPoolExecutor] = None,
) -> pl.DataFrame:
    try:
        import polars as pl
    except ImportError:
        raise missing_dependency_exception("chalkpy[polars]")
    features = [ensure_feature(x) for x in features]
    if version == DatasetVersion.DATASET_WRITER:
        return _load_dataset_from_chalk_writer(uris, features)
    if version in (
        DatasetVersion.BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES,
        DatasetVersion.BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES_V2,
    ):
        return _load_dataset_bigquery(uris, features, executor, version)
    if version in {DatasetVersion.COMPUTE_RESOLVER_OUTPUT_V1}:
        return _load_dataset_bigquery(uris, features, executor, version)
    raise ValueError(
        f"The dataset version ({version}) is not supported by this installed version of the Chalk client. Please upgrade your chalk client and try again."
    )



import datetime
import unittest

from chalk.client import FeatureResult
from chalk.client.client_impl import OnlineQueryResponseWrapper
from chalk.features import features, has_one


@features
class User:
    id: str
    email: str


@features
class PlaidAccount:
    user_id: str
    user: User = has_one(lambda: PlaidAccount.user_id == User.id)
    bank_name: str


data = [
    FeatureResult(
        field="plaid_account.user.email",
        value="faker@gmail.com",
        error=None,
        ts=datetime.datetime(2022, 12, 29, 22, 31, 10, tzinfo=datetime.timezone.utc),
    ),
    FeatureResult(
        field="plaid_account.user.id",
        value="IuiTTRfsldkj0878650X6Kn9mjDE73",
        error=None,
        ts=datetime.datetime(2022, 12, 29, 22, 31, 10, tzinfo=datetime.timezone.utc),
    ),
    FeatureResult(
        field="plaid_account.bank_name",
        value="Wells Fargo",
        error=None,
        ts=datetime.datetime(2022, 12, 29, 22, 31, 10, tzinfo=datetime.timezone.utc),
    ),
]

data_response = OnlineQueryResponseWrapper(data=data, errors=[])

repr_result = """                      field  ...                        ts
0  plaid_account.user.email  ... 2022-12-29 22:31:10+00:00
1     plaid_account.user.id  ... 2022-12-29 22:31:10+00:00
2   plaid_account.bank_name  ... 2022-12-29 22:31:10+00:00

[3 rows x 4 columns]"""

html_result = """<div><stylescoped>.dataframetbodytrth:only-of-type{vertical-align:middle;}.dataframetbodytrth{vertical-align:top;}.dataframetheadth{text-align:right;}</style><tableborder="1"class="dataframe"><thead><trstyle="text-align:right;"><th></th><th>field</th><th>value</th><th>error</th><th>ts</th></tr></thead><tbody><tr><th>0</th><td>plaid_account.user.email</td><td>faker@gmail.com</td><td>None</td><td>2022-12-2922:31:10+00:00</td></tr><tr><th>1</th><td>plaid_account.user.id</td><td>IuiTTRfsldkj0878650X6Kn9mjDE73</td><td>None</td><td>2022-12-2922:31:10+00:00</td></tr><tr><th>2</th><td>plaid_account.bank_name</td><td>WellsFargo</td><td>None</td><td>2022-12-2922:31:10+00:00</td></tr></tbody></table></div>"""


class TestClientStringRepresentation(unittest.TestCase):
    def test_OQRW_data_repr(self):
        self.assertEqual("".join(repr_result.split()), "".join(repr(data_response).split()))

    def test_OQRW_data_repr_html(self):
        self.assertEqual("".join(html_result.split()), "".join(data_response._repr_html_().split()))



from datetime import datetime

from chalk.features import feature_time, features, unwrap_feature


@features
class FeaturesClassWithoutTimestamp:
    pass


def test_features_class_without_timestamp():
    ts_feature_wrapped = getattr(FeaturesClassWithoutTimestamp, "__chalk_observed_at__")
    ts_feature = unwrap_feature(ts_feature_wrapped)
    assert ts_feature.attribute_name == "__chalk_observed_at__"
    assert ts_feature.name == "__chalk_observed_at__"
    assert ts_feature.namespace == "features_class_without_timestamp"
    assert ts_feature.typ is not None
    assert ts_feature.typ.underlying == datetime
    assert [ts_feature] == FeaturesClassWithoutTimestamp.features
    assert FeaturesClassWithoutTimestamp.__chalk_ts__ is ts_feature


@features
class FeaturesClassWithNamedTs:
    ts: datetime


def test_features_class_with_named_ts():
    ts_feature_wrapped = FeaturesClassWithNamedTs.ts
    ts_feature = unwrap_feature(ts_feature_wrapped)
    assert ts_feature.attribute_name == "ts"
    assert ts_feature.name == "ts"
    assert ts_feature.namespace == "features_class_with_named_ts"
    assert ts_feature.typ is not None
    assert ts_feature.typ.underlying == datetime
    assert [ts_feature] == FeaturesClassWithNamedTs.features
    assert FeaturesClassWithNamedTs.__chalk_ts__ is ts_feature


@features
class FeaturesClassWithCustomTsName:
    ts_custom_name: datetime = feature_time()


def test_features_class_with_custom_ts_name():
    ts_feature_wrapped = FeaturesClassWithCustomTsName.ts_custom_name
    ts_feature = unwrap_feature(ts_feature_wrapped)
    assert ts_feature.attribute_name == "ts_custom_name"
    assert ts_feature.name == "ts_custom_name"
    assert ts_feature.namespace == "features_class_with_custom_ts_name"
    assert ts_feature.typ is not None
    assert ts_feature.typ.underlying == datetime
    assert [ts_feature] == FeaturesClassWithCustomTsName.features
    assert FeaturesClassWithCustomTsName.__chalk_ts__ is ts_feature



from typing import Type, TypeVar, cast

from typing_extensions import Annotated

T = TypeVar("T")


class PrimaryMeta(type):
    def __getitem__(self, item: Type[T]) -> Type[T]:
        return cast(Type[T], Annotated[item, "__chalk_primary__"])


Primary = PrimaryMeta("Primary", (object,), {})



import contextlib
from typing import Any, Optional

from sqlalchemy import (
    BIGINT,
    BINARY,
    BLOB,
    BOOLEAN,
    CHAR,
    DATETIME,
    FLOAT,
    INTEGER,
    SMALLINT,
    TEXT,
    TIMESTAMP,
    VARBINARY,
    VARCHAR,
    BigInteger,
    Boolean,
    Date,
    DateTime,
    Float,
    Integer,
    LargeBinary,
    SmallInteger,
    String,
    Text,
    Time,
)
from sqlalchemy.engine.url import URL
from sqlalchemy.sql import Select

from chalk.integrations.named import load_integration_variable
from chalk.sql._internal.sql_source import BaseSQLSource, validate_dtypes_for_efficient_execution
from chalk.sql.finalized_query import FinalizedChalkQuery
from chalk.utils.missing_dependency import missing_dependency_exception

_SUPPORTED_SQLALCHEMY_TYPES_FOR_PA_QUERYING = (
    BigInteger,
    Boolean,
    BINARY,
    BLOB,
    LargeBinary,
    Float,
    Integer,
    Time,
    String,
    Text,
    VARBINARY,
    DateTime,
    Date,
    SmallInteger,
    BIGINT,
    BOOLEAN,
    CHAR,
    DATETIME,
    FLOAT,
    INTEGER,
    SMALLINT,
    TEXT,
    TIMESTAMP,
    VARCHAR,
)


class SnowflakeSourceImpl(BaseSQLSource):
    def __init__(
        self,
        *,
        name: Optional[str] = None,
        account_identifier: Optional[str] = None,
        warehouse: Optional[str] = None,
        user: Optional[str] = None,
        password: Optional[str] = None,
        db: Optional[str] = None,
        schema: Optional[str] = None,
        role: Optional[str] = None,
    ):
        try:
            import snowflake  # noqa
            import snowflake.sqlalchemy  # noqa
        except ModuleNotFoundError:
            raise missing_dependency_exception("chalkpy[snowflake]")
        del snowflake  # unused

        self.account_identifier = account_identifier or load_integration_variable(
            integration_name=name, name="SNOWFLAKE_ACCOUNT_ID"
        )
        self.warehouse = warehouse or load_integration_variable(integration_name=name, name="SNOWFLAKE_WAREHOUSE")
        self.user = user or load_integration_variable(integration_name=name, name="SNOWFLAKE_USER")
        self.password = password or load_integration_variable(integration_name=name, name="SNOWFLAKE_PASSWORD")
        self.db = db or load_integration_variable(integration_name=name, name="SNOWFLAKE_DATABASE")
        self.schema = schema or load_integration_variable(integration_name=name, name="SNOWFLAKE_SCHEMA")
        self.role = role or load_integration_variable(integration_name=name, name="SNOWFLAKE_ROLE")

        super(SnowflakeSourceImpl, self).__init__()

    def local_engine_url(self) -> URL:
        query = {
            k: v
            for k, v in (
                {
                    "database": self.db,
                    "schema": self.schema,
                    "warehouse": self.warehouse,
                    "role": self.role,
                }
            ).items()
            if v is not None
        }
        return URL.create(
            drivername="snowflake",
            username=self.user,
            password=self.password,
            host=self.account_identifier,
            query=query,
        )

    def execute_query_efficient(self, finalized_query: FinalizedChalkQuery, connection: Optional[Any] = None):
        # this import is safe because the only way we end up here is if we have a valid SnowflakeSource constructed,
        # which already gates this import
        import snowflake.connector

        if isinstance(finalized_query.query, Select):
            validate_dtypes_for_efficient_execution(finalized_query.query, _SUPPORTED_SQLALCHEMY_TYPES_FOR_PA_QUERYING)

        with (
            snowflake.connector.connect(
                user=self.user,
                account=self.account_identifier,
                password=self.password,
                warehouse=self.warehouse,
                schema=self.schema,
                database=self.db,
            )
            if connection is None
            else contextlib.nullcontext(connection)
        ) as con:
            assert con is not None
            sql, positional_params, named_params = self.compile_query(finalized_query)
            assert len(positional_params) == 0, "using named param style"
            with con.cursor() as cursor:
                res = cursor.execute(sql, named_params)
                assert res is not None

                arrows = cursor.fetch_arrow_all()
                assert arrows is not None
                return arrows



import logging

import chalk.utils.log_with_context


class _UserLoggerFilter(logging.Filter):
    def filter(self, record: logging.LogRecord) -> bool:
        record.is_user_logger = True
        return True


# Named to "chalk.clogging.chalk_logger" for backwards compatibility
# The name must begin with `chalk` so it will be picked up by the logging filters
chalk_logger = chalk.utils.log_with_context.get_logger("chalk.clogging.chalk_logger")
chalk_logger.addFilter(_UserLoggerFilter())



import pytest
from pydantic import BaseModel

from chalk.features import DataFrame, features
from chalk.streams import KafkaSource, Windowed, stream, windowed

try:
    import duckdb
except ImportError:
    duckdb = None


try:
    import polars as pl
except ImportError:
    pl = None


@features
class StoreFeatures:
    id: str
    purchases: Windowed[float] = windowed("10m", "20m")


class KafkaMessage(BaseModel):
    purchase_id: str
    store_id: str
    amount: float


source = KafkaSource(bootstrap_server="server", topic="topic")


@stream(source=source)
def fn(messages: DataFrame[KafkaMessage]) -> DataFrame[StoreFeatures.id, StoreFeatures.purchases]:
    return f"""
        select store_id as id, sum(amount) as purchases
        from {messages}
        group by 1
    """


@pytest.mark.skipif(duckdb is None or pl is None, reason="duckdb is not installed or polars is not installed")
def test_runs_sql():
    assert pl is not None
    expected = DataFrame(
        pl.DataFrame(
            {
                "store_features.id": ["store1", "store2"],
                "store_features.purchases": [10, 5],
            },
        )
    )
    actual = fn(
        DataFrame(
            pl.DataFrame(
                {
                    "store_id": ["store1", "store2", "store1"],
                    "amount": [4, 5, 6],
                }
            ),
            pydantic_model=KafkaMessage,
        )
    )

    # polars.testing.assert_frame_equal(actual, expected)
    assert expected.to_pyarrow() == actual.to_pyarrow()



from datetime import datetime

from chalk.features import feature_time, features, has_one


@features
class NoFunFeatures:
    id: str


@features
class FunFeatures:
    id: str
    nope: str
    single_parent: "NoFunFeatures" = has_one(lambda: FunFeatures.nope == NoFunFeatures.id)
    ts = feature_time()


def test_dictify_features():
    now = datetime.now()
    assert {"fun_features.nope": "hello", "fun_features.ts": now} == dict(FunFeatures(nope="hello", ts=now))
    assert {"fun_features.nope": "hello"} == dict(FunFeatures(nope="hello"))
    assert {"fun_features.nope": "hello"} == dict(FunFeatures(nope="hello", single_parent=NoFunFeatures(id="a")))
    assert {} == dict(FunFeatures())



import re
from codecs import open
from os import path
from typing import List

import pkg_resources
from setuptools import find_packages, setup


def read(name: str) -> str:
    repo_root = path.abspath(path.dirname(__file__))
    with open(path.join(repo_root, name)) as f:
        return f.read()


def read_requirements(name: str) -> List[str]:
    return [str(r) for r in pkg_resources.parse_requirements(read(name))]


def get_version() -> str:
    version_file = "chalk/_version.py"
    match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", read(version_file), re.M)
    assert match is not None, f"Unable to find version string in {version_file}"
    return match.group(1)


extra_requires = {
    "base": [],
    "bigquery": ["sqlalchemy-bigquery==1.5.0"],
    "dev": read_requirements("requirements-dev.txt"),
    "postgresql": ["psycopg2>=2.9.4"],
    "polars": ["polars[timezone]>=0.14.21,<0.16,!=0.15.12"],
    "snowflake": [
        "snowflake-connector-python>=2.8.0",
        "snowflake-sqlalchemy>=1.4.2",
    ],
    "sqlite": [],  # Deprecated; here for backwards compatibility
    "redshift": [
        "sqlalchemy-redshift>=0.8.11",
        "redshift_connector>=2.0.909",
    ],
    "mysql": ["PyMySQL>=1.0.2"],
    "sql": ["duckdb==0.6.0", "sqlglot"],
}
# Excluding dev requirements from "all" so they are not included for userland installs of chalkpy[all]
extra_requires["all"] = [dep for (key, group_deps) in extra_requires.items() for dep in group_deps if key != "dev"]

if __name__ == "__main__":
    setup(
        version=get_version(),
        name="chalkpy",
        author="Chalk AI, Inc.",
        description="Python SDK for Chalk",
        long_description=read("README.md"),
        long_description_content_type="text/markdown",
        python_requires=">=3.8.0",
        url="https://chalk.ai",
        packages=find_packages(exclude=("tests.*",)),
        install_requires=read_requirements("requirements.txt"),
        extras_require=extra_requires,
        include_package_data=True,
        package_data={"chalk": ["py.typed"]},
        classifiers=[
            # Trove classifiers
            # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python",
            "Topic :: Scientific/Engineering :: Artificial Intelligence",
            "Topic :: Scientific/Engineering :: Information Analysis",
            "Topic :: Software Development :: Code Generators",
            "Topic :: Software Development :: Libraries :: Application Frameworks",
            "Topic :: Software Development :: Libraries :: Python Modules",
        ],
        entry_points={"console_scripts": [f"chalkpy=chalk.cli:cli"]},
        setup_requires=[
            "setuptools_scm",
        ],
    )
